@@ -13,6 +13,7 @@ from flask import url_for
 from flask import jsonify
 
 from leselys.externals import opml
+from xml.sax.saxutils import escape as escape_xml
 
 # Unicode python 2-3
 if sys.version < '3':
@@ -46,11 +47,11 @@ def get_datetime(unparsed_date):
 
 
 def get_dicttime(parsed_date):
-    return {'year': parsed_date[0],
-            'month': parsed_date[1],
-            'day': parsed_date[2],
-            'hour': parsed_date[3],
-            'min': parsed_date[4]}
+    return {'year': parsed_date.year,
+            'month': parsed_date.month,
+            'day': parsed_date.day,
+            'hour': parsed_date.hour,
+            'min': parsed_date.minute}
 
 
 # Decorator for webapp
@@ -115,7 +116,7 @@ def cached(timeout=5 * 60, key='view/%s'):
 
 def retrieve_feeds_from_opml(opml_raw):
     result = []
-    feeds = opml.from_string(opml_raw.encode("ascii", "ignore"))
+    feeds = opml.from_string(opml_raw)
     for outline in feeds:
         if len(outline) > 0:
             for feed in outline:
@@ -129,7 +130,7 @@ def retrieve_feeds_from_opml(opml_raw):
 
 def export_to_opml():
     storage = leselys.core.storage
-    header = """<?xml version="1.0" encoding="UTF-8"?>
+    header = """<?xml version="1.0"?>
 <opml version="1.0">
   <head>
     <title>Leselys feeds export</title>
@@ -144,10 +145,10 @@ def export_to_opml():
     <outline text="%s"
         title="%s" type="rss"
         xmlUrl="%s" htmlUrl="%s"/>""" % (
-            feed['title'],
-            feed['title'],
-            feed['url'],
-            feed['url'])
+            escape_xml(feed['title']),
+            escape_xml(feed['title']),
+            escape_xml(feed['url']),
+            escape_xml(feed['url']))
 
     return header + body + footer
 
@@ -167,13 +168,13 @@ def run_retention(delta_day, story_before_retention):
     for feed in storage.get_feeds():
         print("=> %s" % feed['title'].encode('utf-8'))
 
-        stories = storage.get_stories(feed['_id'])
+        stories = storage.get_stories(feed['_id'], 'unreaded', 0, 0)
         nb_stories = len(stories)
         for story in stories:
             if nb_stories <= story_before_retention:
                 print('   | No many stories, so I keept it.')
                 break
-            delta = datetime.datetime.now() - get_datetime(story['last_update'])
+            delta = datetime.datetime.now() - story['last_update']
             print('  | %s (%s days)' % (story['_id'], delta.days))
             if delta.days > delta_day:
                 print('    | deleted.')
