@@ -45,21 +45,24 @@ class Retriever(threading.Thread):
         # self.feed is raw parsed feed
         self.data = feed['entries']
         self.do_retention = do_retention
+
         if feed_from_db:
-            self.title = feed_from_db['title']
+            self.title = feed_from_db.get('title')
         else:
             self.title = feed['feed']['title']
 
     def run(self):
         # This feed comes from database
         feed = storage.get_feed_by_title(self.title)
+        feed_id = feed.get('_id')
 
         for entry in self.data:
-            title = entry['title']
-            link = entry['link']
+            title = entry.get('title')
+            link = entry.get('link')
+            guid = entry.get('guid') or entry.get('id') or title
 
-            if storage.get_story_by_title(feed['_id'], title):
-                storage.remove_story(storage.get_story_by_title(feed['_id'], title)['_id'])
+            if storage.get_story_by_guid(feed_id, guid):
+                storage.remove_story(storage.get_story_by_guid(feed_id, guid).get('_id'))
 
             try:
                 description = entry['content'][0]['value']
@@ -67,28 +70,27 @@ class Retriever(threading.Thread):
                 description = entry['summary']
 
             if entry.get('updated_parsed'):
-                last_update = get_dicttime(entry.updated_parsed)
+                last_update = get_datetime(entry.updated_parsed)
             else:
-                last_update = get_dicttime(datetime.datetime.now().timetuple())
+                last_update = datetime.datetime.now()
             if entry.get('published_parsed', False):
-                published = get_dicttime(entry.published_parsed)
-                published_datetime = get_datetime(entry.published_parsed)
+                published = get_datetime(entry.published_parsed)
             else:
-                published = get_dicttime(datetime.datetime.now().timetuple())
-                published_datetime = datetime.datetime.now()
+                published = datetime.datetime.now()
 
             if self.do_retention:
-                delta = datetime.datetime.now() - published_datetime
+                delta = datetime.datetime.now() - published
                 if delta.days > int(config.get('worker', 'retention')):
                     continue
 
             storage.add_story({
                 'title': title,
+                'guid': guid,
                 'link': link,
                 'description': description,
                 'published': published,
                 'last_update': last_update,
-                'feed_id': feed['_id'],
+                'feed_id': feed_id,
                 'read': False})
 
 
@@ -99,47 +101,44 @@ class Refresher(threading.Thread):
     def __init__(self, feed):
         threading.Thread.__init__(self)
         self.feed = feed
-        self.feed_id = u(feed['_id'])
+        self.feed_id = feed.get('_id')
+        self.feed_title = feed.get('title')
 
     def run(self):
-        self.data = feedparser.parse(self.feed['url'])
+        self.data = feedparser.parse(self.feed.get('url'))
 
-        # Update title if it change (yes some guys change it...)
-        if self.data.feed['title'] != self.feed['title']:
-            self.feed['title'] = self.data.feed['title']
-            storage.update_feed(self.feed['_id'], copy.copy(self.feed))
+        # Update title if it change
+        if self.data.feed.get('title') != self.feed_title:
+            self.feed['title'] = self.data.feed.get('title')
+            self.feed_title = self.feed['title']
+            storage.update_feed(self.feed_id, copy.copy(self.feed))
 
-        local_update = get_datetime(self.feed['last_update'])
+        local_update = self.feed.get('last_update')
         remote_update = False
         if self.data.feed.get('updated_parsed'):
             remote_update = get_datetime(self.data.feed.updated_parsed)
-            remote_update_raw = get_dicttime(self.data.feed.updated_parsed)
         if self.data.get('updated_parsed'):
             if remote_update:
                 if get_datetime(self.data.updated_parsed) > remote_update:
                     remote_update = get_datetime(self.data.updated_parsed)
-                    remote_update_raw = get_dicttime(self.data.updated_parsed)
         if self.data.feed.get('published_parsed'):
             if remote_update:
                 if get_datetime(self.data.feed.published_parsed) > remote_update:
                     remote_update = get_datetime(self.data.feed.published_parsed)
-                    remote_update_raw = get_dicttime(self.data.feed.published_parsed)
         if self.data.get('published_parsed'):
             if remote_update:
-                if get_datetime(self.data.published_parsed > remote_update):
+                if get_datetime(self.data.published_parsed) > remote_update:
                     remote_update = get_datetime(self.data.published_parsed)
-                    remote_update_raw = get_dicttime(self.data.published_parsed)
 
         if not remote_update:
             remote_update = datetime.datetime.now()
-            remote_update_raw = get_dicttime(remote_update.timetuple())
 
         if remote_update > local_update:
-            print('!! %s is outdated.' % self.feed['title'].encode('utf-8'))
+            print('!! %s is outdated.' % self.feed_title.encode('utf-8'))
             readed = []
-            for entry in storage.get_stories(self.feed['_id']):
-                if entry['read']:
-                    readed.append(entry['title'])
+            for entry in storage.get_stories(self.feed_id, "unreaded", 0, 0):
+                if entry.get('read'):
+                    readed.append(entry.get('guid'))
 
             if len(self.data.entries) <= int(config.get('worker', 'story_before_retention')):
                 do_retention = False
@@ -150,17 +149,17 @@ class Refresher(threading.Thread):
             retriever.start()
             retriever.join()
 
-            for entry in readed:
-                if storage.get_story_by_title(self.feed['_id'], entry):
-                    entry = storage.get_story_by_title(self.feed['_id'], entry)
+            for entry_guid in readed:
+                if storage.get_story_by_guid(self.feed_id, entry_guid):
+                    entry = storage.get_story_by_guid(self.feed_id, entry_guid)
                     entry['read'] = True
                     storage.update_story(entry['_id'], copy.copy(entry))
 
-            self.feed['last_update'] = remote_update_raw
+            self.feed['last_update'] = remote_update
             storage.update_feed(self.feed_id, self.feed)
 
         else:
-            print('=> %s is up-to-date.' % self.feed['title'].encode('utf-8'))
+            print('=> %s is up-to-date.' % self.feed_title.encode('utf-8'))
 
 #########################################################################
 # Reader object
@@ -214,15 +213,15 @@ class Reader(object):
         feed_id = storage.get_feed_by_title(title)
         if not feed_id:
             if feed.feed.get('updated_parsed'):
-                feed_update = get_dicttime(feed.feed.updated_parsed)
+                feed_update = get_datetime(feed.feed.updated_parsed)
             elif feed.get('updated_parsed'):
-                feed_update = get_dicttime(feed.updated_parsed)
+                feed_update = get_datetime(feed.updated_parsed)
             elif feed.feed.get('published_parsed'):
-                feed_update = get_dicttime(feed.feed.published_parsed)
+                feed_update = get_datetime(feed.feed.published_parsed)
             elif feed.get('published_parsed'):
-                feed_update = get_dicttime(feed.published_parsed)
+                feed_update = get_datetime(feed.published_parsed)
             else:
-                feed_update = get_dicttime(datetime.datetime.now().timetuple())
+                feed_update = datetime.datetime.now()
 
             feed_id = storage.add_feed({'url': url,
                                         'title': title,
@@ -247,79 +246,62 @@ class Reader(object):
         storage.remove_feed(feed_id)
         return {"success": True, "output": "Feed removed"}
 
-    def get(self, feed_id=False, order_type="user", start=0, end=100):
+    def get(self, feed_id=False, feed_type="combined-ffed", order_type="user", start=0, stop=50):
+        # Special feeds
         if not feed_id:
-            if order_type == "user" or order_type not in ['unreaded', 'published']:
-                order_type = storage.get_setting('all_items_ordering')
-                if not order_type:
-                    storage.set_setting('all_items_ordering', 'unreaded')
-                    order_type = storage.get_setting('all_items_ordering')
+            if feed_type == "combined-feed":
+                if order_type == "user" or order_type not in ['unreaded', 'published']:
+                    order_type = storage.get_feed_setting('combined-feed' , 'ordering')
+                    if not order_type:
+                        storage.set_feed_setting('combined-feed', 'ordering', 'unreaded')
+                        order_type = 'unreaded'
+                    else:
+                        order_type = order_type['value']
+            elif feed_type == "stared-feed":
+                if order_type == "user" or order_type not in ['unreaded', 'published']:
+                    order_type = storage.get_feed_setting('stared-feed', 'ordering')
+                    if not order_type:
+                        storage.set_feed_setting('stared-feed', 'ordering', 'unreaded')
+                        order_type = 'unreaded'
+                    else:
+                        order_type = order_type['value']
+            else:
+                raise Exception('Unknown feed type (%s)' % feed_type)
+        # Normal feed
         else:
             if order_type == "user" or order_type not in ['unreaded', 'published']:
-                setting_order_type = storage.get_feed_setting(feed_id, 'ordering')
-                if not setting_order_type:
+                order_type = storage.get_feed_setting(feed_id, 'ordering')
+                if not order_type:
                     storage.set_feed_setting(feed_id, 'ordering', 'unreaded')
                     order_type = 'unreaded'
                 else:
-                    order_type = setting_order_type['value']
+                    order_type = order_type['value']
 
+        # Get stories
         if not feed_id:
-            stories = storage.all_stories()
+            if feed_type == "combined-feed":
+                stories = storage.all_stories(order_type, start, stop)
+            elif feed_type == "stared-feed":
+                stories = storage.all_stared(order_type, start, stop)
+            else:
+                raise Exception('Unknown feed type (%s)' % feed_type)
         else:
-            stories = storage.get_stories(feed_id)
+            stories = storage.get_stories(feed_id, order_type, start, stop)
 
+        length = storage.get_feed_unread_count(feed_id)
         res = []
-        entries = []
-        if order_type == 'unreaded':
-            for entry in stories:
-                story = {
-                    "title": entry['title'],
-                    "_id": entry['_id'],
-                    "read": entry['read'],
-                    'last_update': entry['last_update']}
-                if not feed_id:
-                    story['feed_id'] = entry['feed_id']
+        for story in stories:
+            story['last_update'] = get_dicttime(story['last_update'])
+            story['published'] = get_dicttime(story['published'])
+            res.append(story)
 
-                res.append(story)
-
-            # Readed
-            readed = []
-            for entry in res:
-                if entry['read']:
-                    readed.append(entry)
-            readed.sort(key=lambda r: get_datetime(r['last_update']), reverse=True)
-            # Unread
-            unreaded = []
-            for entry in res:
-                if not entry['read']:
-                    unreaded.append(entry)
-            unreaded.sort(key=lambda r: get_datetime(r['last_update']),
-                          reverse=True)
-
-            entries = unreaded + readed
-
-        elif order_type == 'published':
-            for entry in stories:
-                story = {
-                    "title": entry['title'],
-                    "_id": entry['_id'],
-                    "read": entry['read'],
-                    'last_update': entry['published']}
-                if not feed_id:
-                    story['feed_id'] = entry['feed_id']
-
-                res.append(story)
-
-            res.sort(key=lambda r: get_datetime(r['last_update']), reverse=True)
-            entries = res
-
-        return {'entries': entries, 'ordering': order_type}
+        return {'entries': res, 'ordering': order_type, 'detail': {'start': start, 'stop': stop, 'length': length}}
 
     def get_combined_feed(self):
-        order_type = storage.get_setting('all_items_ordering')
+        order_type = storage.get_feed_setting('combined-feed', 'ordering')
         if not order_type:
-            storage.set_setting('all_items_ordering', 'unreaded')
-            order_type = storage.get_setting('all_items_ordering')
+            storage.set_feed_setting('combined-feed', 'ordering', 'unreaded')
+            order_type = storage.get_feed_setting('combined-feed', 'ordering')
         return {'title': 'All stories',
                 'id': 'combined_feed',
                 'counter': self.get_unread(),
@@ -328,16 +310,17 @@ class Reader(object):
     def get_feeds(self):
         feeds = []
         for feed in storage.get_feeds():
-            ordering = storage.get_feed_setting(feed['_id'], 'ordering')
+            feed_id = feed.get('_id')
+            ordering = storage.get_feed_setting(feed_id, 'ordering')
             if not ordering:
-                storage.set_feed_setting(feed['_id'], 'ordering', 'unreaded')
-                ordering = storage.get_feed_setting(feed['_id'], 'ordering')
+                storage.set_feed_setting(feed_id, 'ordering', 'unreaded')
+                ordering = storage.get_feed_setting(feed_id, 'ordering')
 
             ordering = ordering['value']
 
-            feeds.append({'title': feed['title'],
-                          'id': feed['_id'],
-                          'url': feed['url'],
+            feeds.append({'title': feed.get('title'),
+                          'id': feed_id,
+                          'url': feed.get('url'),
                           'counter': self.get_unread(feed['_id']),
                           'ordering': ordering
                           })
@@ -353,27 +336,22 @@ class Reader(object):
 
     def get_unread(self, feed_id=False):
         if not feed_id:
-            stories = 0
-            for story in storage.all_stories():
-                if not story['read']:
-                    stories += 1
-            return stories
-        return len(storage.get_feed_unread(feed_id))
+            return storage.get_feed_unread_count()
+        return storage.get_feed_unread_count(feed_id)
 
     def mark_all_read(self, feed_id):
-        stories = storage.get_stories(feed_id)
-        if not stories:
-            return {'success': False, "content": "Feed not found"}
-        for story in storage.get_stories(feed_id):
+        if feed_id == "combined-feed":
+            stories = storage.all_stories('unreaded', 0, 0)
+        else:
+            stories = storage.get_stories(feed_id, 'unreaded', 0, 0)
+
+        for story in stories:
             story['read'] = True
             storage.update_story(story['_id'], copy.copy(story))
         return {'success': True, "content": "All feed stories updated"}
 
     def mark_all_unread(self, feed_id):
-        stories = storage.get_stories(feed_id)
-        if not stories:
-            return {'success': False, "content": "Feed not found"}
-        for story in storage.get_stories(feed_id):
+        for story in storage.get_stories(feed_id, 'unreaded', 0, 0):
             story['read'] = False
             storage.update_story(story['_id'], copy.copy(story))
         return {'success': True, "content": "All feed stories updated"}
@@ -385,6 +363,8 @@ class Reader(object):
         """
         story = storage.get_story_by_id(story_id)
         if story['read']:
+            story['published'] = get_dicttime(story['published'])
+            story['last_update'] = get_dicttime(story['last_update'])
             return {'success': False,
                     'output': 'Story already readed',
                     'content': story}
@@ -392,12 +372,22 @@ class Reader(object):
         # Save read state before update it for javascript counter in UI
         story['read'] = True
         storage.update_story(story['_id'], copy.copy(story))
+        
+        story['published'] = get_dicttime(story['published'])
+        story['last_update'] = get_dicttime(story['last_update'])
+        
         return {'success': True, 'content': story}
 
     def unread(self, story_id):
         story = storage.get_story_by_id(story_id)
         if not story['read']:
+            story['published'] = get_dicttime(story['published'])
+            story['last_update'] = get_dicttime(story['last_update'])
             return {'success': False, 'output': 'Story already unreaded'}
         story['read'] = False
         storage.update_story(story['_id'], copy.copy(story))
+
+        story['published'] = get_dicttime(story['published'])
+        story['last_update'] = get_dicttime(story['last_update'])
+
         return {'success': True, 'content': story}
