@@ -44,11 +44,10 @@ api.setFeedSetting = function(feedId, settingKey, settingValue, callback) {
 }
 
 api.importOPML = function(file, callback) {
-    return ajaxRequest({
+    return sendFile({
         url: '/api/import/opml',
-        method: 'POST',
         params: {
-            file: file
+            fileInput: file
         },
         callback: callback
     });
@@ -76,9 +75,15 @@ api.getHome = function(callback) {
     });
 }
 
-api.getFeed = function(feedId, callback) {
+api.getFeed = function(feedId, start, stop, callback) {
+    var start = start || 0;
+    var stop = stop || 50;
     return ajaxRequest({
         url: '/api/get/' + feedId,
+        params: {
+            start: start,
+            stop: stop
+        },
         method: 'GET',
         callback: callback
     });
