@@ -19,7 +19,7 @@ def run(config_path, args):
     broker = core.config.get('worker', 'broker')
     interval = core.config.get('worker', 'interval')
     retention = core.config.get('worker', 'retention')
-
+    story_before_retention = core.config.get('worker', 'story_before_retention')
 
     celery = Celery('tasks', broker=broker)
 
@@ -39,11 +39,10 @@ def run(config_path, args):
         'retention-job': {
             'task': 'leselys.worker.run_retention',
             'schedule': timedelta(days=1),
-            'args': (retention,)
+            'args': (int(retention), int(story_before_retention))
         }
     }
     celery.conf.INSTALLED_APPS = ('tasks.refresh_all', 'tasks.run_retention')
 
     print('Args: %s' % ' '.join(args))
     celery.start(args)
-
